/*
 * A server that just receive what the clients send and throws away it.
 * Author: Catalin(ux) M. BOIE
 */

#define _GNU_SOURCE

#include <Conn.h>

/* Global variables */
static unsigned short		debug = 9;
static short			ipv4 = 1, ipv6 = 1;
static int			port = 9000;

static unsigned long long	bytes = 0;
static unsigned long long	errors = 0;
static unsigned long long	latency = 0;


static void s_accept_error(struct Conn *C)
{
	printf("Cannot accept a new connection via %llu!\n",
		Conn_getid(C));
	errors++;
}

static void s_data(struct Conn *C)
{
	/* You can connect to server and send 'quit' command */
	if (Conn_strcasestr(C, "quit"))
		Conn_stop();

	/* throw away all data */
	bytes += Conn_qlen(C);
	Conn_eatall(C);
}

static void s_error(struct Conn *C)
{
	printf("%s id=%llu (%s)\n",
		__FUNCTION__, Conn_getid(C), Conn_strerror());
	errors++;
}

static void s_close(struct Conn *C)
{
	latency += Conn_lifetime(C);
}

int main(void)
{
	struct Conn *C = NULL;
	int ret;

	printf("\tPort=%d\n", port);
	printf("\tDebug=%d\n", debug);
	printf("\tipv4=%s ipv6=%s\n", (ipv4 == 1) ? "on" : "off", (ipv6 == 1) ? "on" : "off");
	printf("\tsys: %s\n", Conn_sys());

	Conn_default_ibuf = 4096;
	Conn_default_obuf = 4096;

	Conn_debug(NULL, debug);

	ret = Conn_init(0);
	if (ret == -1) {
		printf("%s", Conn_strerror());
		return 1;
	}

	Conn_data_cb = s_data;
	Conn_error_cb = s_error;
	Conn_accept_error_cb = s_accept_error;
	Conn_close_cb = s_close;

	if (ipv4 == 1) {
		printf("Trying to register IPv4 socket...\n");
		C = Conn_alloc();
		if (!C) {
			printf("Cannot alloc socket [%s].\n", Conn_strerror());
			errors++;
		} else {
			Conn_set_socket_domain(C, PF_INET);
			Conn_set_socket_type(C, SOCK_STREAM);
			Conn_set_socket_bind_addr(C, "127.0.0.1");
			Conn_set_socket_bind_port(C, port);
			ret = Conn_commit(C);
			if (ret != 0) {
				printf("Cannot bind on ipv4 socket [%s].\n",
					Conn_strerror());
				errors++;
			}
		}
	}

	if (ipv6 == 1) {
		printf("Trying to register IPv6 socket...\n");
		C = Conn_alloc();
		if (!C) {
			printf("Cannot alloc socket [%s].\n", Conn_strerror());
			errors++;
		} else {
			Conn_set_socket_domain(C, PF_INET6);
			Conn_set_socket_type(C, SOCK_STREAM);
			Conn_set_socket_bind_addr(C, "::1");
			Conn_set_socket_bind_port(C, port);
			ret = Conn_commit(C);
			if (ret != 0) {
				printf("Cannot bind on ipv6 socket [%s].\n",
					Conn_strerror());
				errors++;
			}
		}
	}

	while (1) {
		/* -1 we do not have any other business but wait for an event */
		ret = Conn_poll(-1);
		if (ret == -1) {
			printf("Error calling Conn_poll [%s]!\n",
				Conn_strerror());
			return 1;
		} else if (ret == 0)
			break;
		printf("%d event(s). Conn_no=%u.\n",
			ret, Conn_no);
	}

	Conn_shutdown();

	printf("Finish. Errors=%llu, bytes=%llu, latency=%llums.\n\n",
		errors, bytes, latency);

	return 0;
}
