#define _GNU_SOURCE

#include <errno.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/poll.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <time.h>
#include <arpa/inet.h>
#include <stdarg.h>

#include <Conn.h>

/* Global variables */
static unsigned short	debug = 20;

static FILE	*Logf = NULL;
static char	*log_file = "xbind.log";

static void s_error(struct Conn *C)
{
	Log(1, "%s id=%llu (%s)\n",
		__FUNCTION__, Conn_getid(C), Conn_strerror());
}

int main(void)
{
	struct Conn *C;
	int ret, i, family, protocol;
	char *addr;
	char *status;

	Logf = fopen(log_file, "w");
	if (!Logf) {
		fprintf(stderr, "Cannot open log file [%s] [%s]\n",
			log_file, strerror(errno));
		return 1;
	}
	if (debug > 0)
		setlinebuf(Logf);

	/* set library output debug */
	Conn_debug(Logf, debug);

	ret = Conn_init(0);
	if (ret == -1) {
		Log(0, "%s", Conn_strerror());
		return 1;
	}

	Conn_error_cb = s_error;

	addr = "127.0.0.1";
	family = Conn_addr_family(addr);
	protocol = SOCK_STREAM;

	i = 0;
	while (i++ < 100) {
		C = Conn_socket_addr(family, protocol, addr, 65533);
		if (!C) {
			Log(0, "Cannot bind [%s]!\n", Conn_strerror());
			continue;
		}
		Conn_close(C);

		ret = Conn_poll(0);
		Log(0, "Conn_pol returned %d.\n", ret);
	}

	status = Conn_status(0);
	Log(0, "STATUS: %s", status);
	free(status);

	Conn_shutdown();

	Log(0, "Finish!\n\n");

	fclose(Logf);

	return 0;
}
